@@ -4,7 +4,6 @@ use 5.006;
 use strict;
 use warnings;
 
-#use Attribute::Generator;
 use namespace::autoclean;
 use Moose;
 use Storable qw( dclone );
@@ -35,11 +34,11 @@ Hypothesis::API - Wrapper for the hypothes.is web (HTTP) API.
 
 =head1 VERSION
 
-Version 0.08
+Version 0.09
 
 =cut
 
-our $VERSION = '0.08';
+our $VERSION = '0.09';
 
 =head1 SYNOPSIS
 
@@ -79,10 +78,8 @@ Search functionality (no login needed):
     my @annotations;
     while ( my $item = $iter->() ) {
         push @annotations, $item;
-        
     }
 
-
 =head1 EXPORT
 
 Currently nothing.
@@ -257,7 +254,7 @@ Interface to DELETE /api/annotations/<id>
 
 Given an annotation id, returns a boolean value indicating whether or
 not the annotation for that id has been successfully delete (1 = yes,
-0 = no);
+0 = no).
 
 =cut
 
@@ -267,6 +264,13 @@ sub delete_id {
         warn "No id given to delete.\n";
         return 0;
     }
+    my $h = HTTP::Headers->new;
+    $h->header(
+        'content-type' => 'application/json;charset=UTF-8', 
+        'x-csrf-token' => $self->csrf_token,
+        'X-Annotator-Auth-Token' => $self->token, 
+    );
+    $self->ua->default_headers( $h );
     my $url = URI->new( "${\$self->api_url}/annotations/$id" );
     my $response = $self->ua->delete( $url );
     my $json_content;
@@ -274,6 +278,7 @@ sub delete_id {
         $json_content = $json->decode($response->content);
     } catch {
         warn "Trouble decoding JSON: $_\n";
+        warn $response->content;
         return 0;
     };
     if (not $success) {
@@ -395,6 +400,7 @@ as defined in the hypothes.is HTTP API:
  * limit
  * offset
  * uri
+ * uri.parts
  * text
  * quote
  * user
@@ -451,8 +457,6 @@ sub search {
             $response = $self->ua->get( $url );
             $json_content = $json->decode($response->content);
             @annotation_buff = @{$json_content->{ 'rows' }};
-            #TODO: Add a search_id check for last element; 
-            #TODO  warn and return undef otherwise?
             if (@annotation_buff > $page_size) {
                 if (defined $next_buf_start) {
                     while ($next_buf_start->{'id'} ne $annotation_buff[0]->{'id'}) {
@@ -477,6 +481,56 @@ sub search {
 
 }
 
+
+=head2 update_id($id, \%payload)
+
+Interface to PUT /api/annotations/<id>
+
+Updates the annotation for a given annotation id if id is defined and
+the user is authenticated and has update permissions. Takes a payload
+as described for 'search'. Only fields specified in the new payload
+are altered; other existing fields should remain unchanged.
+
+Returns a boolean value indicating whether or not the annotation for
+that id has been successfully delete (1 = yes, 0 = no).
+
+=cut
+
+sub update_id {
+    my ($self, $id, $payload) = @_;
+    if (not defined $id) {
+        die "Can only call update if given an id.";
+    }
+    my $data = $json->encode($payload);
+    my $h = HTTP::Headers->new;
+    $h->header(
+        'content-type' => 'application/json;charset=UTF-8', 
+        'x-csrf-token' => $self->csrf_token,
+        'X-Annotator-Auth-Token' => $self->token, 
+    );
+    $self->ua->default_headers( $h );
+    my $url = URI->new( "${\$self->api_url}/annotations/$id" );
+    my $response = $self->ua->put( $url, Content => $data );
+    my $json_content = $json->decode($response->content);
+    my $content_type = ref($json_content);
+    if ($content_type eq "HASH") {
+        if (defined $json_content->{'updated'}) {
+            if ($json_content->{'updated'}) {
+                return 1;
+            } elsif (not $json_content->{'deleted'}) {
+                return 0;
+            } else { # Never reached in current implementation
+                warn "unexpected update status: ${\$json_content->{'updated'}}";
+                return 0;
+            }
+        } else {
+            die "Received unexpected object: no 'updated' entry present.";
+        }
+    } else {
+        die "Got $content_type; expected an ARRAY or HASH.";
+    }
+}
+
 =head1 AUTHOR
 
 Brandon E. Barker, C<< <brandon.barker at cornell.edu> >>
